#include <ccan/cdump/cdump.h>
#include <ccan/cdump/cdump.c>
#include <ccan/cdump/cdump_internal.h>
#include <ccan/cdump/test/test_header.h>
#include <ccan/str/str.h>
#include <ccan/tap/tap.h>
#include <stddef.h>
#include <string.h>

/* This is as generated by cdump_parse: see run-parse.c */
#include <ccan/cdump/test/example_generated-decls.h>
#include <ccan/cdump/test/example_generated-defs.h>

static struct test2 *new_test2(const void *ctx, struct test2 *next)
{
	struct test2 *t;

	t = talloc(ctx, struct test2);
	t->x1 = 1;
	t->foo = talloc_strdup(t, "foo");
	memset(t->fstring, 'a', 20);
	t->dlen = 100;
	t->dfoo = talloc_array(t, char, t->dlen);
	memset(t->dfoo, 7, t->dlen);
	t->fvalue = RASBERRY;
	t->next = next;
	return t;
}

static bool test2eq(const struct test2 *a, const struct test2 *b)
{
	if (a->x1 != b->x1)
		return false;
	if (!a->foo != !b->foo)
		return false;
	if (a->foo && !streq(a->foo, b->foo))
		return false;
	if (memcmp(a->fstring, b->fstring, 20) != 0)
		return false;
	if (a->dlen != b->dlen)
		return false;
	if (!a->dfoo != !b->dfoo)
		return false;
	if (memcmp(a->dfoo, b->dfoo, a->dlen) != 0)
		return false;
	if (a->fvalue != b->fvalue)
		return false;
	if (!a->next != !b->next)
		return false;
	if (a->next)
		return test2eq(a->next, b->next);
	return true;
}

int main(void)
{
	struct test2 *a, *b;
	char *astr;
	char *toplevel = talloc_strdup(NULL, "toplevel");

	plan_tests(9);
	a = new_test2(toplevel, NULL);
	astr = cdump_bundle(toplevel, cdump_struct_test2, a);
	ok1(astr);
	ok1(talloc_find_parent_byname(astr, "toplevel") == toplevel);

	b = talloc_zero(toplevel, struct test2);
	ok1(cdump_unbundle(toplevel, cdump_struct_test2, b, astr));
	ok1(test2eq(a, b));

	/* Test chaining. */
	b->x1++;
	a->next = b;

	astr = cdump_bundle(toplevel, cdump_struct_test2, a);
	ok1(astr);
	ok1(talloc_find_parent_byname(astr, "toplevel") == toplevel);

	b = talloc_zero(toplevel, struct test2);
	ok1(cdump_unbundle(toplevel, cdump_struct_test2, b, astr));
	ok1(talloc_find_parent_byname(b, "toplevel") == toplevel);

	ok1(test2eq(a, b));
	talloc_free(toplevel);
	
	return exit_status();
}
